local format = string.format
local L = AceLibrary("AceLocale-2.2"):new("Talented")

local function getChatWrite()
	local edit = DEFAULT_CHAT_FRAME.editBox
	local SendChatMessage = SendChatMessage
	local type = edit:GetAttribute("chatType")
	local lang = edit.language
	if type == "WHISPER" then
		local target = edit:GetAttribute("tellTarget")
		return function (text, ...)
			if text:find("%", 1, true) then text = text:format(...) end
			SendChatMessage(text, type, lang, target)
		end
	elseif type == "CHANNEL" then
		local channel = edit:GetAttribute("channelTarget")
		return function (text, ...)
			if text:find("%", 1, true) then text = text:format(...) end
			SendChatMessage(text, type, lang, channel)
		end
	else
		return function (text, ...)
			if text:find("%", 1, true) then text = text:format(...) end
			SendChatMessage(text, type, lang)
		end
	end
end

function Talented:ExportTemplateToChat()
	local write = getChatWrite()
	write(L["Talented - Template \"%s\" - %s :"], self.template.name, UnitClass("player"))
	for tab, talents in ipairs(self.talents) do
		local branchwritten = false
		for index, talent in ipairs(talents.infos) do
			local value = self.template[tab][index]
			if value > 0 then
				local ranks = talent.ranks
				if not branchwritten then
					write(L["%s :"], talents.name)
					branchwritten = true
				end
				if ranks == 1 then
					write(L["_ %s"], talent.name)
				else
					write(L["_ %s (%d/%d)"], talent.name, value, ranks)
				end
			end
		end
	end
end

if Prat then
	local Prat_Talented = Prat:NewModule(Prat:RequestModuleName("PratTalented"))

	Prat_Talented.defaultDB = {
		on = true,
		colortemplate = true,
		color = { r = 0.1, g = 0.3, b = 1.0 },
	}
	
	Prat_Talented.moduleOptions = {
			name = "Talented",
			desc = "Talented Links options.",
			type = "group",
			args = {
				colortemplate = {
					name = "Color Template",
					desc = "Toggle the Template color on and off.",
					type = "toggle",
					order = 180,
					get = function() return Prat_Talented.db.profile.colortemplate end,
					set = function(v) Prat_Talented.db.profile.colortemplate = v end
				},
				setcolor = {
					name = "Set Color",
					desc = "Change the color of the Template.",
					type = "color",
					order = 190,
					get = function() return Prat_Talented.db.profile.color.r, Prat_Talented.db.profile.color.g, Prat_Talented.db.profile.color.b end,
					set = function(r, g, b, a) Prat_Talented.db.profile.color.r, Prat_Talented.db.profile.color.g, Prat_Talented.db.profile.color.b = r, g, b end,
					disabled = function() if not Prat_Talented.db.profile.colortemplate then return true else return false end end,
				},
			}
		}

	function Prat_UrlCopy:GetModuleOptions()		
		return self.moduleOptions 
	end

	local cat = Prat.Categories
	Prat_Talented.Categories = { cat.TEXT, cat.FORMATTING,}

	function Prat_Talented:TemplateLinkToChat(template)
		if not template then return end
		local code
		if template.code then
			code = template.code
		else
			local res, value = pcall(Talented.TemplateToString, Talented, template)
			if not res then return end
			code = value
		end
		self:InsertLink(UnitName("player"), template.name, code)
	end
	
	Prat_Talented.chat_options = {
		name = L["Write template link"],
		desc = L["Write a link to the current template in the chat."],
		type = "execute",
		order = 230,
		func = function () Prat_Talented:TemplateLinkToChat(Talented.template) end,
	}
	
	function Prat_Talented:InsertLink(player, name, code)
		local link = format("{template:%s:%s:%s}", player, name:gsub(":", " "), code)
		ChatEdit_InsertLink(link)
	end

	function Prat_Talented:OnModuleEnable()

		Talented.options.args.mode.args.chat = self.chat_options
		
		self:Hook("SetItemRef", true)

		Prat:RegisterPattern({ pattern = "{template:(.-):(.-):(.-)}", matchfunc=function(...) return self:MakeTemplateLink(...) end }, self.name)
	end

	function Prat_Talented:MakeTemplateLink(player, name, code)
		local link = format("|Htemplate:%s:%s:%s|h[Template %s - %s]|h", code, player, name, player, name)
		if self.db.profile.colortemplate then
			local c = self.db.profile.color
			link = format("|cff%02x%02x%02x%s|r", c.r*255, c.g*255, c.b*255, link)
		end
		return link
	end

	function Prat_Talented:OnModuleDisable()
		Talented.options.args.mode.args.chat = nil
		
		Prat:UnregisterAllPatterns(self.name)
	end

	function Prat_Talented:Prat_PreAddMessage(message, frame, t, r, g, b)
		message.MESSAGE = self:ProcessText(message.MESSAGE)
	end

	function Prat_Talented:ProcessText(text)
		self.link_num = 1
		text = self:Decompose(text)
		
		if Prat_Talented.link_num > 1 then
			for k, v in pairs(Prat_Talented.linkTable) do
				text = string.gsub(text, k, v)
				Prat_Talented.linkTable[k]= nil
			end
			
			Prat_Talented.link_num = 1
		end        
		
		return text
	end

	function Prat_Talented:Decompose (chatstring)
		if chatstring ~= nil then
			for _, v in ipairs(self.PatternMap) do         
				chatstring = string.gsub (chatstring, v.pattern, v.matchfunc) 
			end        
		end
		return chatstring
	end

	
	function Prat_Talented:SetItemRef(link, text, button)
--~ 		Talented:Print("SetItemRef(%q) => %q", link, link:sub(1, 8))
		if link:sub(1, 8) == "template" then
			local code, player, name = strsplit(":", link:sub(10, -1))
--~ 			Talented:Print("SetItemRef(%q, %q, %q)", code, player, name)
			
			local res, val, class = pcall(Talented.StringToTemplate, Talented, code)
			if not res then return end
			local _, pclass = UnitClass("player")
			if class ~= pclass then return end

			local template = Talented:ImportFromOther(player, { name = name, code = code})
			ShowUIPanel(Talented.base)
			Talented:SetTemplate(template)
		else 
			self.hooks.SetItemRef(link, text, button)
		end
	end
else
	function Talented:TemplateLinkToChat()
	end
end

